------------------------------------------------------------
-- Credit for original Mod goes to Azam, for his InfoBar mod
-- See http://www.curse-gaming.com/mod.php?addid=741 for the
-- Version that I based this mod off of
--
-- This mod simply strives to do the same, but instead using
-- Informant, and FuBar, instead of LootLink and InfoBar
--
-- The reason for using FuBar, is because it provides a
-- A much cleaner interface for developers than InfoBar, and
-- Informant is used because it has the value of most items
-- In the game out of the box, that way when you get that
-- Random new item, you can know how much you have before
-- Going to a vendor :P
--
-- A note about missing features though, this does not have
-- The difference display mode at all, as I find it spams
-- The screen much too frequently for my tastes, even when
-- In a group on group loot
-- It may be added at a later date though
------------------------------------------------------------

BagWorthFu = FuBarPlugin:new({
		name          = BagWorthFuLocals.NAME,
		description   = BagWorthFuLocals.DESCRIPTION,
		website       = BagWorthFuLocals.WEBSITE,
		version       = "2.0.0",
		releaseDate   = "5/4/2006",
		aceCompatible = 103,
		fuCompatible  = 10000,
		author        = "Guvante",
		email         = "guvante@gmail.com",
		category      = "inventory",
		db            = AceDatabase:new("BagWorthFuDB"),
		defaults      = {
			color = "7fffff",
			ignored = {},
			ignorelimit = 4,
			qualityBasedWorth = {}
		},
		cmd           = AceChatCmd:new(BagWorthFuLocals.COMMANDS, BagWorthFuLocals.CMD_OPTIONS),
		Locale        = BagWorthFuLocals,
	});

function BagWorthFu:Enable()
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "OnPlayerEnteringWorld")
	self:RegisterEvent("PLAYER_LEAVING_WORLD", "OnPlayerLeavingWorld")
	self:RegisterEvent("BAG_UPDATE", "OnEvent");
	self:OnEvent();
end

function BagWorthFu:OnPlayerEnteringWorld()
	self:RegisterEvent("BAG_UPDATE", "OnEvent");
end

function BagWorthFu:OnPlayerLeavingWorld()
	self:UnregisterEvent("BAG_UPDATE");
end

function BagWorthFu:UpdateText()
	self:SetText(self.UpdatedText or "");
end

function BagWorthFu:UpdateTooltip()
	local empty = TRUE;
	for i = 6, 0, -1 do
		if self.data.qualityBasedWorth[i] then
			--Reuse ignorelimit mapping here, since it is the same info we need
			self.tooltip:AddDoubleLine(nil, self.Locale.MAP.IGNORELIMIT[i], FuBarUtils.FormatMoneyFull(self.data.qualityBasedWorth[i], true));
			empty = FALSE;
		end
	end

	if empty then
		FuBarTooltip:AddLine(nil, self.Locale.EMPTYVALUE, 1, 1, 1);
	end
end

function BagWorthFu:OnEvent()
	local itemLink, itemId, itemPrice, itemQuality, itemName;
	self.data.qualityBasedWorth = {};
	for bag = 0, 4, 1 do
		for slot = 1, GetContainerNumSlots(bag), 1 do
			itemLink = GetContainerItemLink(bag, slot);
			--This function returns an itemId as its first value
			itemId, itemQuality, itemName = self:BreakLink(itemLink);
			itemPrice = self:GetPrice(itemName, itemId);
			if (itemPrice and (self:NotIgnore(itemId))) then
				local _, itemCount = GetContainerItemInfo(bag, slot);
				if (itemPrice > 0) and itemCount and (itemCount > 0) then
					self.data.qualityBasedWorth[itemQuality] = (self.data.qualityBasedWorth[itemQuality] or 0) + (itemCount * itemPrice);
				end
			end
		end
	end
	local totalBagWorth = 0;
	for i= 0,6 do
		if i < self.data.ignorelimit then
			totalBagWorth = totalBagWorth + (self.data.qualityBasedWorth[i] or 0);
		end
	end
	--This sets the text of our box to be the formated money we have with the color of the digits with the selected color
	self.UpdatedText = string.gsub(FuBarUtils.FormatMoneyFull(totalBagWorth, true), "ff" .. FuBarUtils.COLOR_HEX_WHITE, "ff" .. self:Color());
	self:Update();
end

function BagWorthFu:BreakLink(itemLink)
	if not itemLink then return end
	--Thanks to the Auctioneer team for this code (Specifically EnhTooltip)
	local _, _, itemQuality, itemId, itemName = string.find(itemLink, "(|c%x+)|Hitem:(%d+):%d+:%d+:%d+|h%[(.-)%]|h|r");
	return tonumber(itemId), self.Locale.COLORTOINDEX[itemQuality], itemName;
end

function BagWorthFu:GetPrice(itemName, itemId)
	if not (itemName and itemId) then return end
	--Check for LootLink
	if KC_Items and KC_Items.common and KC_Items.common:GetItemPrices(itemId) then
		return (KC_Items.common:GetItemPrices(itemId));
	--Check for ItemSync
	elseif ISync and ISync.GetCorID and ISync.GetDB and ISync:GetCoreID(link) and ISync:GetDB(ISync:GetCorID(link), "price") then
		return ISync:GetDB(ISync:GetCorID(link), "price");
	--Check for KC_Items
	elseif ItemLinks and ItemLinks[itemName] and ItemLinks[itemName].p then
		return ItemLinks[itemName].p;
	--Check for Informant
	elseif Informant and Informant.GetItem and Informant.GetItem(itemId) and (Informant.GetItem(itemId)).sell then
		return (Informant.GetItem(itemId)).sell;
	end
	--All else fails, return a FALSE to tell ourselves that we don't have data for that item
	return FALSE;
end

function BagWorthFu:Report()
	local reportTable = {{text=self.Locale.STATUS.IGNORELIMIT, val=self:IgnoreLimit(), map=self.Locale.MAP.IGNORELIMIT},
						 {text=self.Locale.STATUS.COLORCODE, val="|cff" .. self:Color() .. self:Color() .. "|r"}}
	--Add in the items we are ignoring
	for _,v in pairs(self.data.ignored) do
		tinsert(reportTable, {text=self.Locale.STATUS.IGNORE, val=v});
	end

	--Report the basic information
	self.cmd:report(reportTable);
end

--Sets the color code used to display the panel text
function BagWorthFu:SetColor(ColorCode)
	self.data.color = ColorCode;
	self.cmd:status(self.Locale.STATUS.COLORCODE, self:Color());
end

--Sets the color code to the default
function BagWorthFu:SetDefaultColor()
	self:SetColor(self.defaults.color);
end

--Grabs the color we want to replace the money's white color with
function BagWorthFu:Color()
	return self.data.color;
end

--Returns true if the item is not being ignored, and false otherwise
function BagWorthFu:NotIgnore(itemId)
	return not self.data.ignored[itemId];
end

--Toggles ignoring the selected item, and displays a status message to the user
function BagWorthFu:IgnoreItem(itemLink)
	local itemId, _, itemName = self:BreakLink(itemLink);
	if self.data.ignored[itemId] then
		self.data.ignored[itemId] = FALSE;
		self.cmd:result(self.Locale.STATUS.NOTIGNORE, itemName);
	else
		self.data.ignored[itemId] = itemName;
		self.cmd:result(self.Locale.STATUS.IGNORE, itemName);
	end

	--Trigger the OnEvent function so our GUI is updated
	self:OnEvent();
end

--Clears all ignoring data
function BagWorthFu:ClearIgnoredItems()
	self.cmd:msg(self.Locale.STATUS.CLEARIGNORE);
	self.data.ignored = {};
	self:OnEvent();
end

--Return the current ignore limit
function BagWorthFu:IgnoreLimit()
	return self.data.ignorelimit;
end

--Display the new ignore limit
function BagWorthFu:IgnoreLimitDisplay()
	self.cmd:status(self.Locale.STATUS.IGNORELIMIT, self:IgnoreLimit(), self.Locale.MAP.IGNORELIMIT);
end

--Set the ignore limit to a certain value, one function for each possible value
function BagWorthFu:IgnoreLimitNone()
	self.data.ignorelimit = 7;
	self:IgnoreLimitDisplay();
end

function BagWorthFu:IgnoreLimitArtifact()
	self.data.ignorelimit = 6;
	self:IgnoreLimitDisplay();
end

function BagWorthFu:IgnoreLimitLegendary()
	self.data.ignorelimit = 5;
	self:IgnoreLimitDisplay();
end

function BagWorthFu:IgnoreLimitEpic()
	self.data.ignorelimit = 4;
	self:IgnoreLimitDisplay();
end

function BagWorthFu:IgnoreLimitRare()
	self.data.ignorelimit = 3;
	self:IgnoreLimitDisplay();
end

function BagWorthFu:IgnoreLimitUncommon()
	self.data.ignorelimit = 2;
	self:IgnoreLimitDisplay();
end

function BagWorthFu:IgnoreLimitCommon()
	self.data.ignorelimit = 1;
	self:IgnoreLimitDisplay();
end

BagWorthFu:RegisterForLoad();